@@ -8,10 +8,19 @@ Chisel schema type model
 from collections import Counter, defaultdict
 from datetime import date, datetime, timezone
 from decimal import Decimal
+import importlib.resources
+import json
 from math import isnan, isinf
+import os
 from uuid import UUID
 
 
+# Read the type model JSON resource
+with importlib.resources.path('chisel', 'schema.py') as schema_py_path:
+    with open(os.path.join(os.path.dirname(schema_py_path), 'static', 'typeModel.json'), 'r') as type_model_json_file:
+        _TYPE_MODEL = json.load(type_model_json_file)
+
+
 def get_type_model():
     """
     Get a copy of the Chisel type model
@@ -726,463 +735,3 @@ def _validate_types_type(errors, types, type_, attr, type_name, member_name):
                 attr_value = f'{attr[attr_key]:.6f}'.rstrip('0').rstrip('.')
                 attr_text = f'{_ATTR_TO_TEXT[attr_key]} {attr_value}'
                 error(f'Invalid attribute {attr_text!r}')
-
-
-# The type model's type model
-_TYPE_MODEL = {
-    'Action': {
-        'struct': {
-            'name': 'Action',
-            'doc': ' A JSON web service API',
-            'members': [
-                {
-                    'name': 'name',
-                    'doc': ' The action name',
-                    'type': {'builtin': 'string'}
-                },
-                {
-                    'name': 'doc',
-                    'doc': ' The documentation markdown text',
-                    'type': {'builtin': 'string'},
-                    'optional': True
-                },
-                {
-                    'name': 'docGroup',
-                    'doc': ' The action@s documentation group name',
-                    'type': {'builtin': 'string'},
-                    'optional': True
-                },
-                {
-                    'name': 'urls',
-                    'doc': ' The action@s URLs',
-                    'type': {'array': {'type': {'user': 'ActionURL'}}},
-                    'attr': {'lenGT': 0},
-                    'optional': True
-                },
-                {
-                    'name': 'path',
-                    'doc': ' The path parameters struct type name',
-                    'type': {'builtin': 'string'},
-                    'optional': True
-                },
-                {
-                    'name': 'query',
-                    'doc': ' The query parameters struct type name',
-                    'type': {'builtin': 'string'},
-                    'optional': True
-                },
-                {
-                    'name': 'input',
-                    'doc': ' The content body struct type name',
-                    'type': {'builtin': 'string'},
-                    'optional': True
-                },
-                {
-                    'name': 'output',
-                    'doc': ' The response body struct type name',
-                    'type': {'builtin': 'string'},
-                    'optional': True
-                },
-                {
-                    'name': 'errors',
-                    'doc': ' The custom error response codes enum type name',
-                    'type': {'builtin': 'string'},
-                    'optional': True
-                }
-            ]
-        }
-    },
-    'ActionURL': {
-        'struct': {
-            'name': 'ActionURL',
-            'doc': ' An action URL model',
-            'members': [
-                {
-                    'name': 'method',
-                    'doc': ' The HTTP method. If not provided, matches all HTTP methods.',
-                    'type': {'builtin': 'string'},
-                    'optional': True
-                },
-                {
-                    'name': 'path',
-                    'doc': ' The URL path. If not provided, uses the default URL path of \'/<actionName>\'.',
-                    'type': {'builtin': 'string'},
-                    'optional': True
-                }
-            ]
-        }
-    },
-    'Array': {
-        'struct': {
-            'name': 'Array',
-            'doc': ' An array type',
-            'members': [
-                {
-                    'name': 'type',
-                    'doc': ' The contained type',
-                    'type': {'user': 'Type'}
-                },
-                {
-                    'name': 'attr',
-                    'doc': ' The contained type@s attributes',
-                    'type': {'user': 'Attributes'},
-                    'optional': True
-                }
-            ]
-        }
-    },
-    'Attributes': {
-        'struct': {
-            'name': 'Attributes',
-            'doc': ' A type or member@s attributes',
-            'members': [
-                {
-                    'name': 'nullable',
-                    'doc': ' If true, the value may be null',
-                    'type': {'builtin': 'bool'},
-                    'optional': True
-                },
-                {
-                    'name': 'eq',
-                    'doc': ' The value is equal',
-                    'type': {'builtin': 'float'},
-                    'optional': True
-                },
-                {
-                    'name': 'lt',
-                    'doc': ' The value is less than',
-                    'type': {'builtin': 'float'},
-                    'optional': True
-                },
-                {
-                    'name': 'lte',
-                    'doc': ' The value is less than or equal to',
-                    'type': {'builtin': 'float'},
-                    'optional': True
-                },
-                {
-                    'name': 'gt',
-                    'doc': ' The value is greater than',
-                    'type': {'builtin': 'float'},
-                    'optional': True
-                },
-                {
-                    'name': 'gte',
-                    'doc': ' The value is greater than or equal to',
-                    'type': {'builtin': 'float'},
-                    'optional': True
-                },
-                {
-                    'name': 'lenEq',
-                    'doc': ' The length is equal to',
-                    'type': {'builtin': 'int'},
-                    'optional': True
-                },
-                {
-                    'name': 'lenLT',
-                    'doc': ' The length is less-than',
-                    'type': {'builtin': 'int'},
-                    'optional': True
-                },
-                {
-                    'name': 'lenLTE',
-                    'doc': ' The length is less than or equal to',
-                    'type': {'builtin': 'int'},
-                    'optional': True
-                },
-                {
-                    'name': 'lenGT',
-                    'doc': ' The length is greater than',
-                    'type': {'builtin': 'int'},
-                    'optional': True
-                },
-                {
-                    'name': 'lenGTE',
-                    'doc': ' The length is greater than or equal to',
-                    'type': {'builtin': 'int'},
-                    'optional': True
-                }
-            ]
-        }
-    },
-    'BuiltinType': {
-        'enum': {
-            'name': 'BuiltinType',
-            'doc': ' The built-in type enumeration',
-            'values': [
-                {
-                    'name': 'string',
-                    'doc': ' The string type'
-                },
-                {
-                    'name': 'int',
-                    'doc': ' The integer type'
-                },
-                {
-                    'name': 'float',
-                    'doc': ' The float type'
-                },
-                {
-                    'name': 'bool',
-                    'doc': ' The boolean type'
-                },
-                {
-                    'name': 'date',
-                    'doc': ' A date formatted as an ISO-8601 date string'
-                },
-                {
-                    'name': 'datetime',
-                    'doc': ' A date/time formatted as an ISO-8601 date/time string'
-                },
-                {
-                    'name': 'uuid',
-                    'doc': ' A UUID formatted as string'
-                },
-                {
-                    'name': 'object',
-                    'doc': ' An object of any type'
-                }
-            ]
-        }
-    },
-    'Dict': {
-        'struct': {
-            'name': 'Dict',
-            'doc': ' A dictionary type',
-            'members': [
-                {
-                    'name': 'type',
-                    'doc': ' The contained key type',
-                    'type': {'user': 'Type'}
-                },
-                {
-                    'name': 'attr',
-                    'doc': ' The contained key type@s attributes',
-                    'type': {'user': 'Attributes'},
-                    'optional': True
-                },
-                {
-                    'name': 'keyType',
-                    'doc': ' The contained value type',
-                    'type': {'user': 'Type'},
-                    'optional': True
-                },
-                {
-                    'name': 'keyAttr',
-                    'doc': ' The contained value type@s attributes',
-                    'type': {'user': 'Attributes'},
-                    'optional': True
-                }
-            ]
-        }
-    },
-    'Enum': {
-        'struct': {
-            'name': 'Enum',
-            'doc': ' An enumeration type',
-            'members': [
-                {
-                    'name': 'name',
-                    'doc': ' The enum type name',
-                    'type': {'builtin': 'string'}
-                },
-                {
-                    'name': 'doc',
-                    'doc': ' The documentation markdown text',
-                    'type': {'builtin': 'string'},
-                    'optional': True
-                },
-                {
-                    'name': 'values',
-                    'doc': ' The enumeration values',
-                    'type': {'array': {'type': {'user': 'EnumValue'}}},
-                    'attr': {'lenGT': 0},
-                    'optional': True
-                }
-            ]
-        }
-    },
-    'EnumValue': {
-        'struct': {
-            'name': 'EnumValue',
-            'doc': ' An enumeration type value',
-            'members': [
-                {
-                    'name': 'name',
-                    'doc': ' The value string',
-                    'type': {'builtin': 'string'}
-                },
-                {
-                    'name': 'doc',
-                    'doc': ' The documentation markdown text',
-                    'type': {'builtin': 'string'},
-                    'optional': True
-                }
-            ]
-        }
-    },
-    'Struct': {
-        'struct': {
-            'name': 'Struct',
-            'doc': ' A struct type',
-            'members': [
-                {
-                    'name': 'name',
-                    'doc': ' The struct type name',
-                    'type': {'builtin': 'string'}
-                },
-                {
-                    'name': 'doc',
-                    'doc': ' The documentation markdown text',
-                    'type': {'builtin': 'string'},
-                    'optional': True
-                },
-                {
-                    'name': 'members',
-                    'doc': ' The struct members',
-                    'type': {'array': {'type': {'user': 'StructMember'}}},
-                    'attr': {'lenGT': 0},
-                    'optional': True
-                },
-                {
-                    'name': 'union',
-                    'doc': ' If true, the struct is a union and exactly one of the optional members is present',
-                    'type': {'builtin': 'bool'},
-                    'optional': True
-                }
-            ]
-        }
-    },
-    'StructMember': {
-        'struct': {
-            'name': 'StructMember',
-            'doc': ' A struct member',
-            'members': [
-                {
-                    'name': 'name',
-                    'doc': ' The member name',
-                    'type': {'builtin': 'string'}
-                },
-                {
-                    'name': 'doc',
-                    'doc': ' The documentation markdown text',
-                    'type': {'builtin': 'string'},
-                    'optional': True
-                },
-                {
-                    'name': 'type',
-                    'doc': ' The member type',
-                    'type': {'user': 'Type'}
-                },
-                {
-                    'name': 'attr',
-                    'doc': ' The member type attributes',
-                    'type': {'user': 'Attributes'},
-                    'optional': True
-                },
-                {
-                    'name': 'optional',
-                    'doc': ' If true, the member is optional and may not be present',
-                    'type': {'builtin': 'bool'},
-                    'optional': True
-                }
-            ]
-        }
-    },
-    'Type': {
-        'struct': {
-            'name': 'Type',
-            'doc': ' Union representing a member type',
-            'union': True,
-            'members': [
-                {
-                    'name': 'builtin',
-                    'doc': ' A built-in type',
-                    'type': {'user': 'BuiltinType'}
-                },
-                {
-                    'name': 'array',
-                    'doc': ' An array type',
-                    'type': {'user': 'Array'}
-                },
-                {
-                    'name': 'dict',
-                    'doc': ' A dictionary type',
-                    'type': {'user': 'Dict'}
-                },
-                {
-                    'name': 'user',
-                    'doc': ' A user type name',
-                    'type': {'builtin': 'string'}
-                }
-            ]
-        }
-    },
-    'Typedef': {
-        'struct': {
-            'name': 'Typedef',
-            'doc': ' A typedef type',
-            'members': [
-                {
-                    'name': 'name',
-                    'doc': ' The typedef type name',
-                    'type': {'builtin': 'string'}
-                },
-                {
-                    'name': 'doc',
-                    'doc': ' The documentation markdown text',
-                    'type': {'builtin': 'string'},
-                    'optional': True
-                },
-                {
-                    'name': 'type',
-                    'doc': ' The typedef@s type',
-                    'type': {'user': 'Type'}
-                },
-                {
-                    'name': 'attr',
-                    'doc': ' The typedef@s type attributes',
-                    'type': {'user': 'Attributes'},
-                    'optional': True
-                }
-            ]
-        }
-    },
-    'Types': {
-        'typedef': {
-            'name': 'Types',
-            'doc': ' Map of user type name to user type model',
-            'type': {'dict': {'type': {'user': 'UserType'}}},
-            'attr': {'lenGT': 0}
-        }
-    },
-    'UserType': {
-        'struct': {
-            'name': 'UserType',
-            'doc': ' A user type',
-            'union': True,
-            'members': [
-                {
-                    'name': 'enum',
-                    'doc': ' An enumeration type',
-                    'type': {'user': 'Enum'}
-                },
-                {
-                    'name': 'struct',
-                    'doc': ' A struct type',
-                    'type': {'user': 'Struct'}
-                },
-                {
-                    'name': 'typedef',
-                    'doc': ' A type definition',
-                    'type': {'user': 'Typedef'}
-                },
-                {
-                    'name': 'action',
-                    'doc': ' A JSON web API (not reference-able)',
-                    'type': {'user': 'Action'}
-                }
-            ]
-        }
-    }
-}
